@@ -165,6 +165,7 @@ GType ctk_clocks_get_type(void)
             sizeof(CtkClocks),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_object_type = g_type_register_static
@@ -792,7 +793,6 @@ static void overclocking_state_toggled(GtkWidget *widget, gpointer user_data)
 {
     CtkClocks *ctk_object = CTK_CLOCKS(user_data);
     gboolean enabled;
-    ReturnStatus ret;
     int value;
     gint result;
     
@@ -835,10 +835,10 @@ static void overclocking_state_toggled(GtkWidget *widget, gpointer user_data)
 
     
     /* Update the server */
-    
-    ret = NvCtrlSetAttribute(ctk_object->handle,
-                             NV_CTRL_GPU_OVERCLOCKING_STATE, value);
-    
+
+    NvCtrlSetAttribute(ctk_object->handle,
+                       NV_CTRL_GPU_OVERCLOCKING_STATE, value);
+
     /* Update the GUI */
     
     overclocking_state_update_gui(ctk_object);
@@ -1249,18 +1249,17 @@ static void reset_clocks_clicked(GtkWidget *widget, gpointer user_data)
 static void detect_clocks_clicked(GtkWidget *widget, gpointer user_data)
 {
     CtkClocks *ctk_object = CTK_CLOCKS(user_data);
-    ReturnStatus ret;
     gint result;
-    
+
 
     if ( ctk_object->probing_optimal ) {
 
         /* Stop the test for optimal clock freqs */
-        ret = NvCtrlSetAttribute(ctk_object->handle,
-                                 NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION,
-                                 NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_CANCEL);
+        NvCtrlSetAttribute(ctk_object->handle,
+                           NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION,
+                           NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_CANCEL);
     } else {
-        
+
         /* User must hit the OK button to start the testing */
 
         gtk_widget_show_all(ctk_object->detect_dialog);
@@ -1272,9 +1271,9 @@ static void detect_clocks_clicked(GtkWidget *widget, gpointer user_data)
         case GTK_RESPONSE_ACCEPT:
 
             /* Start the test for optimal clock freqs */
-            ret = NvCtrlSetAttribute(ctk_object->handle,
-                                     NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION,
-                                     NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_START);
+            NvCtrlSetAttribute(ctk_object->handle,
+                               NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION,
+                               NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_START);
             break;
 
         case GTK_RESPONSE_REJECT:
