@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -30,7 +25,6 @@
 #include "red_xpm.h"
 #include "green_xpm.h"
 #include "blue_xpm.h"
-#include "ctkbanner.h"
 
 #include "ctkcurve.h"
 #include "ctkscale.h"
@@ -84,7 +78,7 @@ static gfloat
 get_attribute_channel_value (CtkColorCorrection *, gint, gint);
 
 static void
-flush_attribute_channel_values (CtkColorCorrection *, gint, gint, gfloat);
+flush_attribute_channel_values (CtkColorCorrection *, gint, gint);
 
 static void
 ctk_color_correction_class_init(CtkColorCorrectionClass *);
@@ -109,6 +103,8 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define GREEN      GREEN_CHANNEL_INDEX
 #define BLUE       BLUE_CHANNEL_INDEX
 
+#define ALL_CHANNELS_INDEX 3
+
 #define CONTRAST   (CONTRAST_INDEX - CONTRAST_INDEX)
 #define BRIGHTNESS (BRIGHTNESS_INDEX - CONTRAST_INDEX)
 #define GAMMA      (GAMMA_INDEX - CONTRAST_INDEX)
@@ -146,6 +142,7 @@ GType ctk_color_correction_get_type(
             sizeof (CtkColorCorrection),
             0,    /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_color_correction_type =
@@ -161,10 +158,6 @@ static void
 ctk_color_correction_class_init(CtkColorCorrectionClass
                                 *ctk_color_correction_class)
 {
-    GObjectClass *gobject_class;
-
-    gobject_class = (GObjectClass *) ctk_color_correction_class;
-
     signals[CHANGED] =
         g_signal_new("changed",
                      G_OBJECT_CLASS_TYPE(ctk_color_correction_class),
@@ -173,8 +166,6 @@ ctk_color_correction_class_init(CtkColorCorrectionClass
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
-
-
 }
 
 
@@ -184,10 +175,10 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
 {
     GObject *object;
     CtkColorCorrection *ctk_color_correction;
+    GtkRequisition requisition;
 
     GtkWidget *menu;
     GtkWidget *image;
-    GtkWidget *banner;
     GtkWidget *label;
     GtkWidget *scale;
     GtkWidget *curve;
@@ -198,24 +189,11 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     GtkWidget *rightvbox;
     GtkWidget *hbox;
     GtkWidget *vbox;
-    GtkWidget *button, *confirm_button;
+    GtkWidget *button, *confirm_button, *confirm_label;
     GtkWidget *widget;
     GtkWidget *hsep;
     GtkWidget *eventbox;
-    ReturnStatus ret;
-    gint val;
 
-    /* check if the VidMode extension is present */
-
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_ATTR_EXT_VM_PRESENT, &val);
-    if ((ret != NvCtrlSuccess) || (val == FALSE)) return NULL;
-    
-    /* check if the noScanout mode enabled */
-
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_NO_SCANOUT, &val);
-    if ((ret == NvCtrlSuccess) && (val == NV_CTRL_NO_SCANOUT_ENABLED))
-        return NULL;
-    
     object = g_object_new(CTK_TYPE_COLOR_CORRECTION, NULL);
 
     ctk_color_correction = CTK_COLOR_CORRECTION(object);
@@ -228,16 +206,6 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
 
     gtk_box_set_spacing(GTK_BOX(ctk_color_correction), 10);
 
-    /*
-     * Banner: TOP - LEFT -> RIGHT
-     *
-     * This image serves as a visual reference for basic color_box correction
-     * purposes.
-     */
-
-    banner = ctk_banner_image_new(BANNER_ARTWORK_COLOR);
-    gtk_box_pack_start(GTK_BOX(ctk_color_correction), banner, FALSE, FALSE, 0);
-
     /* create the main hbox and the two main vboxes*/
 
     mainhbox = gtk_hbox_new(FALSE, 10);
@@ -366,7 +334,9 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     button = gtk_button_new_with_label("Reset Hardware Defaults");
     gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
     
-    confirm_button = gtk_button_new_with_label("Confirm Current Changes");
+    confirm_button = gtk_button_new();
+    confirm_label = gtk_label_new("Confirm Current Changes");
+    gtk_container_add(GTK_CONTAINER(confirm_button), confirm_label);
     eventbox = gtk_event_box_new();
     gtk_container_add(GTK_CONTAINER(eventbox), confirm_button);
     gtk_box_pack_end(GTK_BOX(hbox), eventbox, FALSE, FALSE, 5);
@@ -379,6 +349,7 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
                      G_CALLBACK(confirm_button_clicked),
                      (gpointer) ctk_color_correction);
 
+    ctk_color_correction->confirm_label = confirm_label;
     ctk_color_correction->confirm_button = confirm_button;
     ctk_config_set_tooltip(ctk_config, eventbox, __confirm_button_help);
     ctk_config_set_tooltip(ctk_config, button, __resest_button_help);
@@ -500,7 +471,24 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     /* finally, show the widget */
 
     gtk_widget_show_all(GTK_WIDGET(object));
-    
+
+    /*
+     * lock the size of the confirm button, so that it is not resized
+     * when we change the button text later.
+     *
+     * Note: this assumes that the initial size of the button is the
+     * largest size needed for any text placed in the button.  In the
+     * case of the confirm button, this works out:
+     *
+     *  "Confirm Current Changes" <-- initial value
+     *  "%d Seconds to Confirm"
+     */
+
+    gtk_widget_size_request(ctk_color_correction->confirm_label,
+                            &requisition);
+    gtk_widget_set_size_request(ctk_color_correction->confirm_label,
+                                requisition.width, -1);
+
     return GTK_WIDGET(object);
 }
 
@@ -536,6 +524,15 @@ static void option_menu_changed(
             break;
     }
 
+    /*
+     * store the currently selected color channel, so that
+     * adjustment_value_changed() can update the correct channel(s) in
+     * response to slider changes
+     */
+
+    g_object_set_data(G_OBJECT(option_menu), "color_channel",
+                      GINT_TO_POINTER(channel));
+
     for (i = 0; i < 3; i++) {
 
         if (i == 0) {
@@ -549,9 +546,6 @@ static void option_menu_changed(
             attribute = GAMMA_VALUE;
         }
         
-        g_object_set_data(G_OBJECT(option_menu), "color_channel",
-                          GINT_TO_POINTER(channel));
-        
         value = get_attribute_channel_value(ctk_color_correction,
                                             attribute, channel);
 
@@ -566,9 +560,6 @@ static void option_menu_changed(
             (G_OBJECT(adjustment),
              G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
              G_CALLBACK(adjustment_value_changed), NULL);
-    
-        if (channel == ALL_CHANNELS)
-            gtk_adjustment_value_changed(GTK_ADJUSTMENT(adjustment));
     }
 }
 
@@ -583,27 +574,43 @@ static void set_button_sensitive(
 
 /** set_color_state() *******************
  *
- * Stores color state to cur_val[attribute][channel]
- * and prev_val[attribute][channel].
+ * Stores color state to cur_slider_val[attribute][channel]
+ * and prev_slider_val[attribute][channel].
  *
  **/
 
 static void set_color_state(CtkColorCorrection *ctk_color_correction,
-    gint attribute_idx, gint channel_idx,
+    gint attribute_idx, gint channel_mask,
     gfloat value, gboolean all
 )
 {
-    if ( channel_idx != ALL_CHANNELS ) {
-        ctk_color_correction->cur_val
-            [attribute_idx][channel_idx] = value;
-        if ( all ) {
-            ctk_color_correction->prev_val
-                [attribute_idx][channel_idx] = value;
+    if (channel_mask & RED_CHANNEL) {
+        ctk_color_correction->cur_slider_val[attribute_idx][RED] = value;
+        if (all) {
+            ctk_color_correction->prev_slider_val[attribute_idx][RED] = value;
         }
-    } else {
-        gint ch;
-        for ( ch = RED; ch <= BLUE; ch++ ) {
-            set_color_state(ctk_color_correction, attribute_idx, ch, value, all);
+    }
+
+    if (channel_mask & GREEN_CHANNEL) {
+        ctk_color_correction->cur_slider_val[attribute_idx][GREEN] = value;
+        if (all) {
+            ctk_color_correction->prev_slider_val[attribute_idx][GREEN] = value;
+        }
+    }
+
+    if (channel_mask & BLUE_CHANNEL) {
+        ctk_color_correction->cur_slider_val[attribute_idx][BLUE] = value;
+        if (all) {
+            ctk_color_correction->prev_slider_val[attribute_idx][BLUE] = value;
+        }
+    }
+
+    if (channel_mask == ALL_CHANNELS) {
+        ctk_color_correction->cur_slider_val
+            [attribute_idx][ALL_CHANNELS_INDEX] = value;
+        if (all) {
+            ctk_color_correction->prev_slider_val
+                [attribute_idx][ALL_CHANNELS_INDEX] = value;
         }
     }
 } /* set_color_state() */
@@ -624,17 +631,19 @@ static void confirm_button_clicked(
 {
     CtkColorCorrection *ctk_color_correction = CTK_COLOR_CORRECTION(user_data);
 
-    /* Store cur_val[attribute][channel] to prev_val[attribute][channel]. */
-    memcpy (ctk_color_correction->prev_val, ctk_color_correction->cur_val,
-            sizeof(ctk_color_correction->cur_val));
+    /* Store cur_slider_val[attribute][channel] to
+       prev_slider_val[attribute][channel]. */
+    memcpy (ctk_color_correction->prev_slider_val,
+            ctk_color_correction->cur_slider_val,
+            sizeof(ctk_color_correction->cur_slider_val));
     
     /* kill the timer */
     g_source_remove(ctk_color_correction->confirm_timer);
     ctk_color_correction->confirm_timer = 0;
     
     /* Reset confirm button text */
-    gtk_button_set_label(GTK_BUTTON(ctk_color_correction->confirm_button),
-                         "Confirm Current Changes");
+    gtk_label_set_text(GTK_LABEL(ctk_color_correction->confirm_label),
+                       "Confirm Current Changes");
     
     gtk_widget_set_sensitive(ctk_color_correction->confirm_button, FALSE);
 } /* confirm_button_clicked() */
@@ -657,12 +666,9 @@ static void reset_button_clicked(
     set_color_state(ctk_color_correction, GAMMA, ALL_CHANNELS,
                     GAMMA_DEFAULT, TRUE);
 
-    NvCtrlSetColorAttributes(ctk_color_correction->handle,
-                             ctk_color_correction->cur_val[CONTRAST],
-                             ctk_color_correction->cur_val[BRIGHTNESS],
-                             ctk_color_correction->cur_val[GAMMA],
-                             ALL_VALUES | ALL_CHANNELS); 
-    
+    flush_attribute_channel_values(ctk_color_correction,
+                                   ALL_VALUES, ALL_CHANNELS);
+
     option_menu = GTK_OPTION_MENU(ctk_color_correction->option_menu);
     
     if (gtk_option_menu_get_history(option_menu) == 0) {
@@ -684,12 +690,14 @@ static void reset_button_clicked(
                              FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
     /* kill the timer */
-    g_source_remove(ctk_color_correction->confirm_timer);
-    ctk_color_correction->confirm_timer = 0;
+    if (ctk_color_correction->confirm_timer) {
+        g_source_remove(ctk_color_correction->confirm_timer);
+        ctk_color_correction->confirm_timer = 0;
+    }
     
     /* Reset confirm button text */
-    gtk_button_set_label(GTK_BUTTON(ctk_color_correction->confirm_button),
-                         "Confirm Current Changes");
+    gtk_label_set_text(GTK_LABEL(ctk_color_correction->confirm_label),
+                       "Confirm Current Changes");
 }
 
 static void adjustment_value_changed(
@@ -699,7 +707,7 @@ static void adjustment_value_changed(
 {
     CtkColorCorrection *ctk_color_correction;
     gint attribute, channel;
-    gint attribute_idx, channel_idx;
+    gint attribute_idx;
     gfloat value;
     gchar *channel_str, *attribute_str;
 
@@ -745,30 +753,25 @@ static void adjustment_value_changed(
 
     switch (channel) {
     case RED_CHANNEL:
-        channel_idx = RED;
         channel_str = "red ";
         break;
     case GREEN_CHANNEL:
-        channel_idx = GREEN;
         channel_str = "green ";
         break;
     case BLUE_CHANNEL:
-        channel_idx = BLUE;
         channel_str = "blue ";
         break;
     case ALL_CHANNELS:
-        channel_idx = ALL_CHANNELS;
         channel_str = "";
         break;
     default:
         return;
     }
 
-    set_color_state(ctk_color_correction, attribute_idx, channel_idx,
+    set_color_state(ctk_color_correction, attribute_idx, channel,
                     value, FALSE);
     
-    flush_attribute_channel_values(ctk_color_correction,
-                                attribute, channel, value);
+    flush_attribute_channel_values(ctk_color_correction, attribute, channel);
     
     ctk_config_statusbar_message(ctk_color_correction->ctk_config,
                                  "Set %s%s to %f.",
@@ -781,51 +784,54 @@ static gfloat get_attribute_channel_value(CtkColorCorrection
                                           *ctk_color_correction,
                                           gint attribute, gint channel)
 {
-    gfloat values[3] = { 0.0, 0.0, 0.0 };
-    gfloat ignore[3] = { 0.0, 0.0, 0.0 };
-
-    NvCtrlAttributeHandle *handle = ctk_color_correction->handle;
+    int attribute_idx, channel_idx;
 
     switch (attribute) {
         case CONTRAST_VALUE:
-            NvCtrlGetColorAttributes(handle, values, ignore, ignore);
+            attribute_idx = CONTRAST;
             break;
         case BRIGHTNESS_VALUE:
-            NvCtrlGetColorAttributes(handle, ignore, values, ignore);
+            attribute_idx = BRIGHTNESS;
             break;
         case GAMMA_VALUE:
-            NvCtrlGetColorAttributes(handle, ignore, ignore, values);
+            attribute_idx = GAMMA;
             break;
         default:
             return 0.0;
     }
 
     switch (channel) {
-        case ALL_CHANNELS: /* XXX what to do for all channels? */
+        case ALL_CHANNELS:
+            channel_idx = ALL_CHANNELS_INDEX;
+            break;
         case RED_CHANNEL:
-            return values[0];
+            channel_idx = RED_CHANNEL_INDEX;
+            break;
         case GREEN_CHANNEL:
-            return values[1];
+            channel_idx = GREEN_CHANNEL_INDEX;
+            break;
         case BLUE_CHANNEL:
-            return values[2];
+            channel_idx = BLUE_CHANNEL_INDEX;
+            break;
         default:
-            return 0;
+            return 0.0;
     }
+
+    return ctk_color_correction->cur_slider_val[attribute_idx][channel_idx];
 }
 
 static void flush_attribute_channel_values(
     CtkColorCorrection *ctk_color_correction,
     gint attribute,
-    gint channel,
-    gfloat value
+    gint channel
 )
 {
     NvCtrlAttributeHandle *handle = ctk_color_correction->handle;
     
     NvCtrlSetColorAttributes(handle,
-                             ctk_color_correction->cur_val[CONTRAST],
-                             ctk_color_correction->cur_val[BRIGHTNESS],
-                             ctk_color_correction->cur_val[GAMMA],
+                             ctk_color_correction->cur_slider_val[CONTRAST],
+                             ctk_color_correction->cur_slider_val[BRIGHTNESS],
+                             ctk_color_correction->cur_slider_val[GAMMA],
                              attribute | channel);
     
     g_signal_emit(ctk_color_correction, signals[CHANGED], 0);
@@ -867,42 +873,42 @@ static void apply_parsed_attribute_list(
         switch (p->attr & (ALL_VALUES | ALL_CHANNELS)) {
         case (CONTRAST_VALUE | RED_CHANNEL):
             set_color_state(ctk_color_correction, CONTRAST,
-                            RED, p->fval, TRUE); break;
+                            RED_CHANNEL, p->val.f, TRUE); break;
         case (CONTRAST_VALUE | GREEN_CHANNEL):
             set_color_state(ctk_color_correction, CONTRAST,
-                            GREEN, p->fval, TRUE); break;
+                            GREEN_CHANNEL, p->val.f, TRUE); break;
         case (CONTRAST_VALUE | BLUE_CHANNEL):
             set_color_state(ctk_color_correction, CONTRAST,
-                            BLUE, p->fval, TRUE); break;
+                            BLUE_CHANNEL, p->val.f, TRUE); break;
         case (CONTRAST_VALUE | ALL_CHANNELS):
             set_color_state(ctk_color_correction, CONTRAST,
-                            ALL_CHANNELS, p->fval, TRUE); break;
+                            ALL_CHANNELS, p->val.f, TRUE); break;
 
         case (BRIGHTNESS_VALUE | RED_CHANNEL):
             set_color_state(ctk_color_correction, BRIGHTNESS,
-                            RED, p->fval, TRUE); break;
+                            RED_CHANNEL, p->val.f, TRUE); break;
         case (BRIGHTNESS_VALUE | GREEN_CHANNEL):
             set_color_state(ctk_color_correction, BRIGHTNESS,
-                            GREEN, p->fval, TRUE); break;
+                            GREEN_CHANNEL, p->val.f, TRUE); break;
         case (BRIGHTNESS_VALUE | BLUE_CHANNEL):
             set_color_state(ctk_color_correction, BRIGHTNESS,
-                            BLUE, p->fval, TRUE); break;
+                            BLUE_CHANNEL, p->val.f, TRUE); break;
         case (BRIGHTNESS_VALUE | ALL_CHANNELS):
             set_color_state(ctk_color_correction, BRIGHTNESS,
-                            ALL_CHANNELS, p->fval, TRUE); break;
+                            ALL_CHANNELS, p->val.f, TRUE); break;
 
         case (GAMMA_VALUE | RED_CHANNEL):
             set_color_state(ctk_color_correction, GAMMA,
-                            RED, p->fval, TRUE); break;
+                            RED_CHANNEL, p->val.f, TRUE); break;
         case (GAMMA_VALUE | GREEN_CHANNEL):
             set_color_state(ctk_color_correction, GAMMA,
-                            GREEN, p->fval, TRUE); break;
+                            GREEN_CHANNEL, p->val.f, TRUE); break;
         case (GAMMA_VALUE | BLUE_CHANNEL):
             set_color_state(ctk_color_correction, GAMMA,
-                            BLUE, p->fval, TRUE); break;
+                            BLUE_CHANNEL, p->val.f, TRUE); break;
         case (GAMMA_VALUE | ALL_CHANNELS):
             set_color_state(ctk_color_correction, GAMMA,
-                            ALL_CHANNELS, p->fval, TRUE); break;
+                            ALL_CHANNELS, p->val.f, TRUE); break;
 
         default:
             goto next_attribute;
@@ -917,9 +923,9 @@ static void apply_parsed_attribute_list(
 
     if (attr) {
         NvCtrlSetColorAttributes(ctk_color_correction->handle,
-                                 ctk_color_correction->cur_val[CONTRAST],
-                                 ctk_color_correction->cur_val[BRIGHTNESS],
-                                 ctk_color_correction->cur_val[GAMMA],
+                                 ctk_color_correction->cur_slider_val[CONTRAST],
+                                 ctk_color_correction->cur_slider_val[BRIGHTNESS],
+                                 ctk_color_correction->cur_slider_val[GAMMA],
                                  attr);
     }
     
@@ -936,8 +942,8 @@ static void update_confirm_text(CtkColorCorrection *ctk_color_correction)
     gchar *str;
     str = g_strdup_printf("%d Seconds to Confirm",
                           ctk_color_correction->confirm_countdown);
-    gtk_button_set_label(GTK_BUTTON(ctk_color_correction->confirm_button),
-                         str);
+    gtk_label_set_text(GTK_LABEL(ctk_color_correction->confirm_label),
+                       str);
     g_free(str);
 } /* update_confirm_text() */
 
@@ -952,7 +958,8 @@ static gboolean do_confirm_countdown(gpointer data)
 {
     gint attr, ch;
     CtkColorCorrection *ctk_color_correction = (CtkColorCorrection *)(data);
-    unsigned int active_attributes_channels = 0;
+    unsigned int channels = 0;
+    unsigned int attributes = 0;
     GtkOptionMenu *option_menu =
         GTK_OPTION_MENU(ctk_color_correction->option_menu);
 
@@ -964,31 +971,29 @@ static gboolean do_confirm_countdown(gpointer data)
 
     /* Countdown timed out, reset color settings to previous state */
     for (attr = CONTRAST_INDEX; attr <= GAMMA_INDEX; attr++) {
-        for (ch = RED; ch <= BLUE; ch++) {
+        for (ch = RED; ch <= ALL_CHANNELS_INDEX; ch++) {
             /* Check for attribute channel value change. */
-            if (ctk_color_correction->cur_val[attr - CONTRAST_INDEX][ch] !=
-                ctk_color_correction->prev_val[attr - CONTRAST_INDEX][ch]) {
-                ctk_color_correction->cur_val[attr - CONTRAST_INDEX][ch] =
-                    ctk_color_correction->prev_val[attr - CONTRAST_INDEX][ch];
-                active_attributes_channels |= (1 << attr) | (1 << ch);
+            int index = attr - CONTRAST_INDEX;
+            if (ctk_color_correction->cur_slider_val[index][ch] !=
+                ctk_color_correction->prev_slider_val[index][ch]) {
+                ctk_color_correction->cur_slider_val[index][ch] =
+                    ctk_color_correction->prev_slider_val[index][ch];
+                attributes |= (1 << attr);
+                channels |= (1 << ch);
             }
         }
     }
-    if (active_attributes_channels) {
-        NvCtrlSetColorAttributes(ctk_color_correction->handle,
-                                 ctk_color_correction->cur_val[CONTRAST],
-                                 ctk_color_correction->cur_val[BRIGHTNESS],
-                                 ctk_color_correction->cur_val[GAMMA],
-                                 active_attributes_channels);
-        g_signal_emit(ctk_color_correction, signals[CHANGED], 0);
+    if (attributes | channels) {
+        flush_attribute_channel_values(ctk_color_correction,
+                                       attributes, channels);
     }
     
     /* Refresh color correction page for current selected channel. */
     option_menu_changed(option_menu, (gpointer)(ctk_color_correction));
     
     /* Reset confirm button text */
-    gtk_button_set_label(GTK_BUTTON(ctk_color_correction->confirm_button),
-                         "Confirm Current Changes");
+    gtk_label_set_text(GTK_LABEL(ctk_color_correction->confirm_label),
+                       "Confirm Current Changes");
     
     /* Update status bar message */
     ctk_config_statusbar_message(ctk_color_correction->ctk_config,
@@ -1007,18 +1012,36 @@ GtkTextBuffer *ctk_color_correction_create_help(GtkTextTagTable *table)
 {
     GtkTextIter i;
     GtkTextBuffer *b;
-
+    const gchar *title = "X Server Color Correction";
+    
     b = gtk_text_buffer_new(table);
     
     gtk_text_buffer_get_iter_at_offset(b, &i, 0);
     
-    ctk_help_title(b, &i, "X Server Color Correction Help");
+    ctk_help_title(b, &i, "%s Help", title);
 
-    ctk_help_heading(b, &i, "Active Color Channel");
-    ctk_help_para(b, &i, __active_color_help);
+    ctk_color_correction_tab_help(b, &i, title, FALSE /* randr */);
+    
+    ctk_help_heading(b, &i, "Reset Hardware Defaults");
+    ctk_help_para(b, &i, __resest_button_help);
 
-    ctk_help_heading(b, &i, "Brightness, Contrast and Gamma");
-    ctk_help_para(b, &i, "The Brightness, Contrast and Gamma sliders "
+    ctk_help_finish(b);
+
+    return b;
+}
+
+
+void ctk_color_correction_tab_help(GtkTextBuffer *b, GtkTextIter *i,
+                                   const gchar *title,
+                                   gboolean randr)
+{
+    ctk_help_heading(b, i, "Color Correction");
+
+    ctk_help_term(b, i, "Active Color Channel");
+    ctk_help_para(b, i, __active_color_help);
+
+    ctk_help_term(b, i, "Brightness, Contrast and Gamma");
+    ctk_help_para(b, i, "The Brightness, Contrast and Gamma sliders "
                   "allow you to adjust the brightness, contrast, "
                   "or gamma values for the selected color channel(s).  This "
                   "helps you to compensate "
@@ -1029,24 +1052,21 @@ GtkTextBuffer *ctk_color_correction_create_help(GtkTextTagTable *table)
                   "as photographs) when they are displayed on your "
                   "monitor.");
 
-    ctk_help_para(b, &i, "Also, many 3D-accelerated games may appear too "
+    ctk_help_para(b, i, "Also, many 3D-accelerated games may appear too "
                   "dark to play.  Increasing the brightness and/or gamma "
                   "value equally across all channels will make these games "
                   "appear brighter, making them more playable.");
-    
-    ctk_help_para(b, &i, __color_curve_help);
 
-    ctk_help_para(b, &i, "Note that the X Server Color Correction page uses "
-                  "the XF86VidMode extension to manipulate the X screen's "
-                  "color ramps.");
-    
-    ctk_help_heading(b, &i, "Confirm Current Changes");
-    ctk_help_para(b, &i, __confirm_button_help);
-    
-    ctk_help_heading(b, &i, "Reset Hardware Defaults");
-    ctk_help_para(b, &i, __resest_button_help);
+    ctk_help_para(b, i, __color_curve_help);
 
-    ctk_help_finish(b);
+    if (randr) {
+        ctk_help_para(b, i, "The %s tab uses the RandR extension to "
+                      "manipulate an RandR CRTC's gamma ramp.", title);
+    } else {
+        ctk_help_para(b, i, "The %s page uses the XF86VidMode extension "
+                      "to manipulate the X screen's gamma ramps", title);
+    }
 
-    return b;
+    ctk_help_term(b, i, "Confirm Current Changes");
+    ctk_help_para(b, i, __confirm_button_help);
 }
