@@ -44,8 +44,8 @@
 #include "ctkslimm.h"
 #include "ctkgpu.h"
 #include "ctkcolorcorrection.h"
+#include "ctkcolorcorrectionpage.h"
 #include "ctkxvideo.h"
-#include "ctkrandr.h"
 #include "ctkcursorshadow.h"
 #include "ctkopengl.h"
 #include "ctkglx.h"
@@ -57,9 +57,8 @@
 #include "ctkpowersavings.h"
 #include "ctk3dvisionpro.h"
 
-#include "ctkdisplaydevice-crt.h"
 #include "ctkdisplaydevice-tv.h"
-#include "ctkdisplaydevice-dfp.h"
+#include "ctkdisplaydevice.h"
 
 #include "ctkdisplayconfig.h"
 #include "ctkserver.h"
@@ -123,7 +122,8 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
                                 NvCtrlAttributeHandle *handle,
                                 CtkEvent *ctk_event,
                                 GtkTextTagTable *tag_table,
-                                UpdateDisplaysData *data);
+                                UpdateDisplaysData *data,
+                                ParsedAttribute *p);
 
 static void update_display_devices(GtkObject *object, gpointer arg1,
                                    gpointer user_data);
@@ -156,6 +156,7 @@ GType ctk_window_get_type(void)
             sizeof(CtkWindow),
             0,                  /* n_preallocs */
             NULL,               /* instance_init */
+            NULL                /* value_table */
         };
 
         ctk_window_type = g_type_register_static
@@ -175,11 +176,9 @@ GType ctk_window_get_type(void)
 
 static void ctk_window_class_init(CtkWindowClass *ctk_window_class)
 {
-    GObjectClass *gobject_class;
     GtkObjectClass *object_class;
 
     object_class = (GtkObjectClass *) ctk_window_class;
-    gobject_class = (GObjectClass *) ctk_window_class;
 
     parent_class = g_type_class_peek_parent(ctk_window_class);
 
@@ -383,6 +382,16 @@ static gboolean tree_view_key_event(GtkWidget *tree_view, GdkEvent *event,
 } /* tree_view_key_event() */
 
 
+static gboolean has_randr_gamma(NvCtrlAttributeHandle *handle)
+{
+    ReturnStatus ret;
+    int val;
+
+    ret = NvCtrlGetAttribute(handle, NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE, &val);
+
+    return ((ret == NvCtrlSuccess) && (val == 1));
+}
+
 /*
  * ctk_window_new() - create a new CtkWindow widget
  */
@@ -563,7 +572,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     /* create the container widget for the "pages" */
 
     hbox = gtk_hbox_new(FALSE, 0);
-    gtk_paned_pack2(GTK_PANED(hpane), hbox, TRUE, TRUE);
+    gtk_paned_pack2(GTK_PANED(hpane), hbox, TRUE, FALSE);
 
     ctk_window->page_viewer = hbox;
     ctk_window->page = NULL;
@@ -667,15 +676,21 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
             }
         }
 
-        /* color correction */
+        /*
+         * color correction, if RandR per-CRTC color correction is not
+         * available
+         */
 
-        child = ctk_color_correction_new(screen_handle, ctk_config,
-                                         ctk_window->attribute_list,
-                                         ctk_event);
-        if (child) {
-            help = ctk_color_correction_create_help(tag_table);
-            add_page(child, help, ctk_window, &iter, NULL,
-                     "X Server Color Correction", NULL, NULL, NULL);
+        if (!has_randr_gamma(screen_handle)) {
+
+            child = ctk_color_correction_page_new(screen_handle, ctk_config,
+                                                  ctk_window->attribute_list,
+                                                  ctk_event);
+            if (child) {
+                help = ctk_color_correction_page_create_help(tag_table);
+                add_page(child, help, ctk_window, &iter, NULL,
+                         "X Server Color Correction", NULL, NULL, NULL);
+            }
         }
 
         /* xvideo settings  */
@@ -687,15 +702,6 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                      "X Server XVideo Settings", NULL, NULL, NULL);
         }
 
-        /* randr settings */
-
-        child = ctk_randr_new(screen_handle, ctk_config, ctk_event);
-        if (child) {
-            help = ctk_randr_create_help(tag_table, CTK_RANDR(child));
-            add_page(child, help, ctk_window, &iter, NULL,
-                     "Rotation Settings", NULL, NULL, NULL);
-        }
-
         /* cursor shadow */
 
         child = ctk_cursor_shadow_new(screen_handle, ctk_config, ctk_event);
@@ -813,6 +819,12 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_CONFIG_FILE_ATTRIBUTES_FUNC_COLUMN,
                            NULL, -1);
+        gtk_tree_store_set(ctk_window->tree_store, &iter,
+                           CTK_WINDOW_SELECT_WIDGET_FUNC_COLUMN,
+                           ctk_gpu_page_select, -1);
+        gtk_tree_store_set(ctk_window->tree_store, &iter,
+                           CTK_WINDOW_UNSELECT_WIDGET_FUNC_COLUMN,
+                           ctk_gpu_page_unselect, -1);
 
         /* power savings */
 
@@ -863,14 +875,14 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         data->gpu_handle = gpu_handle;
         data->parent_iter = iter;
         data->tag_table = tag_table;
-        
+
         g_signal_connect(G_OBJECT(ctk_event),
                          CTK_EVENT_NAME(NV_CTRL_PROBE_DISPLAYS),
                          G_CALLBACK(update_display_devices),
-                         (gpointer) data);            
-        
-        add_display_devices(ctk_window, &iter, gpu_handle, ctk_event,
-                            tag_table, data);
+                         (gpointer) data);
+
+        add_display_devices(ctk_window, &iter, gpu_handle, ctk_event, tag_table,
+                            data, ctk_window->attribute_list);
     }
 
     /* add the per-vcs (e.g. Quadro Plex) entries into the tree model */
@@ -887,9 +899,9 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
         /* create the vcs entry name */
 
-        ret = NvCtrlGetStringDisplayAttribute(vcs_handle, 0,
-                                              NV_CTRL_STRING_VCSC_PRODUCT_NAME,
-                                              &vcs_product_name);
+        ret = NvCtrlGetStringAttribute(vcs_handle,
+                                       NV_CTRL_STRING_VCSC_PRODUCT_NAME,
+                                       &vcs_product_name);
         if (ret == NvCtrlSuccess && vcs_product_name) {
             vcs_name = g_strdup_printf("VCS %d - (%s)",
                                         NvCtrlGetTargetId(vcs_handle),
@@ -1360,102 +1372,127 @@ void add_special_config_file_attributes(CtkWindow *ctk_window)
  */
 
 static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
-                                NvCtrlAttributeHandle *handle,
-                                CtkEvent *ctk_event,
+                                NvCtrlAttributeHandle *gpu_handle,
+                                CtkEvent *ctk_event_gpu,
                                 GtkTextTagTable *tag_table,
-                                UpdateDisplaysData *data)
+                                UpdateDisplaysData *data,
+                                ParsedAttribute *p)
 {
-    GtkWidget *widget;
     GtkTextBuffer *help;
     ReturnStatus ret;
-    int i, connected, n, mask;
-    char *name;
-    char *type;
-    gchar *title;
-    
-    
-    /* retrieve the connected display device mask */
+    int *pData;
+    int len;
+    int i;
+    NvCtrlAttributeHandle *display_handle;
 
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_CONNECTED_DISPLAYS, &connected);
-    if (ret != NvCtrlSuccess) {
+
+    /* retrieve the list of connected display devices */
+
+    ret = NvCtrlGetBinaryAttribute(gpu_handle, 0,
+                                   NV_CTRL_BINARY_DATA_DISPLAYS_CONNECTED_TO_GPU,
+                                   (unsigned char **)(&pData), &len);
+    if ((ret != NvCtrlSuccess) || (pData[0] <= 0)) {
         return;
     }
 
-    /* count how many display devices are connected */
-
-    for (i = 0, n = 0; i < 24; i++) {
-        if (connected & (1 << i)) n++;
-    }
-    if (n == 0) {
-        return;
-    }
-    
-
-    if (data->display_iters) {
-        free(data->display_iters);
-    }
-    data->display_iters = calloc(n, sizeof(GtkTreeIter));
+    nvfree(data->display_iters);
+    data->display_iters = calloc(pData[0], sizeof(GtkTreeIter));
     data->num_displays = 0;
     if (!data->display_iters) return;
 
 
     /*
-     * create pages for each of the display devices driven by this handle.
+     * create pages for each of the display devices driven by this (gpu)
+     * handle.
      */
 
-    for (i = 0; i < 24; i++) {
-        mask = (1 << i);
-        if (!(mask & connected)) continue;
+    for (i = 0; i < pData[0]; i++) {
+        int display_id = pData[i+1];
+        char *logName;
+        char *typeBaseName;
+        char *typeIdName;
+        GtkWidget *widget;
+        gchar *title;
+        CtkEvent *ctk_event;
+        CtrlHandles *handles = ctk_window->ctk_config->pCtrlHandles;
 
-        type = display_device_mask_to_display_device_name(mask);
-        
-        ret =
-            NvCtrlGetStringDisplayAttribute(handle, mask,
-                                            NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
-                                            &name);
-        
-        if ((ret != NvCtrlSuccess) || (!name)) {
-            title = g_strdup_printf("%s - (Unknown)", type);
+        /* 
+         * Get the ctrl handle that was passed into ctk_main so that updated
+         * backend color slider values, cached in the handle itself, can be 
+         * saved to the RC file when the UI is closed.
+         */
+
+        if (handles) {
+            display_handle = 
+                handles->targets[NV_CTRL_TARGET_TYPE_DISPLAY].t[display_id].h;
         } else {
-            title = g_strdup_printf("%s - (%s)", type, name);
-            XFree(name);
+            display_handle = NULL;
         }
-        free(type);
-        
-        if (mask & CTK_DISPLAY_DEVICE_CRT_MASK) {
-            
-            widget = ctk_display_device_crt_new
-                (handle, ctk_window->ctk_config, ctk_event, mask, title);
-            
-            help = ctk_display_device_crt_create_help
-                (tag_table, CTK_DISPLAY_DEVICE_CRT(widget));
-            
-        } else if (mask & CTK_DISPLAY_DEVICE_TV_MASK) {
-            
-            widget = ctk_display_device_tv_new
-                (handle, ctk_window->ctk_config, ctk_event, mask, title);
-            
-            help = ctk_display_device_tv_create_help
-                (tag_table, CTK_DISPLAY_DEVICE_TV(widget));
-            
-        } else if (mask & CTK_DISPLAY_DEVICE_DFP_MASK) {
-            
-            widget = ctk_display_device_dfp_new
-                (handle, ctk_window->ctk_config, ctk_event, mask, title);
-            
-            help = ctk_display_device_dfp_create_help
-                (tag_table, CTK_DISPLAY_DEVICE_DFP(widget));
-            
-        } else {
-            g_free(title);
+
+        if (!display_handle) {
             continue;
         }
 
-        add_page(widget, help, ctk_window, iter,
-                 &(data->display_iters[data->num_displays]), title,
-                 NULL, NULL, NULL);
+        ctk_event = CTK_EVENT(ctk_event_new(display_handle));
+
+        /* Query display's names */
+        ret = NvCtrlGetStringAttribute(display_handle,
+                                       NV_CTRL_STRING_DISPLAY_NAME_TYPE_BASENAME,
+                                       &typeBaseName);
+        if (ret != NvCtrlSuccess) {
+            NvCtrlAttributeClose(display_handle);
+            continue;
+        }
+        ret = NvCtrlGetStringAttribute(display_handle,
+                                       NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
+                                       &logName);
+        if (ret != NvCtrlSuccess) {
+            logName = NULL;
+        }
+        ret = NvCtrlGetStringAttribute(display_handle,
+                                       NV_CTRL_STRING_DISPLAY_NAME_TYPE_ID,
+                                       &typeIdName);
+        if (ret != NvCtrlSuccess) {
+            typeIdName = NULL;
+        }
+
+        if (!logName && !typeIdName) {
+            title = g_strdup_printf("DPY-%d - (Unknown)", display_id);
+        } else {
+            title = g_strdup_printf("%s - (%s)", typeIdName, logName);
+        }
+        XFree(logName);
+        XFree(typeIdName);
+
+        /* Create the appropriate page for the display */
+        if (strcmp(typeBaseName, "TV") == 0) {
+            widget = ctk_display_device_tv_new
+                (display_handle, ctk_window->ctk_config, ctk_event, title);
+            help = ctk_display_device_tv_create_help
+                (tag_table, CTK_DISPLAY_DEVICE_TV(widget));
+        } else if ((strcmp(typeBaseName, "DFP") == 0) ||
+                   (strcmp(typeBaseName, "CRT") == 0)) {
+            widget = ctk_display_device_new(display_handle,
+                                            ctk_window->ctk_config, ctk_event,
+                                            ctk_event_gpu,
+                                            title, typeBaseName, p);
+            help = ctk_display_device_create_help(tag_table,
+                                                  CTK_DISPLAY_DEVICE(widget));
+        } else {
+            widget = NULL;
+        }
+
+        if (!widget) {
+            NvCtrlAttributeClose(display_handle);
+        } else {
+            add_page(widget, help, ctk_window, iter,
+                     &(data->display_iters[data->num_displays]), title,
+                     NULL, NULL, NULL);
+            data->num_displays++;
+        }
+
+        XFree(typeBaseName);
         g_free(title);
-        data->num_displays++;
     }
 } /* add_display_devices() */
 
@@ -1474,7 +1511,6 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
     UpdateDisplaysData *data = (UpdateDisplaysData *) user_data;
 
     CtkWindow *ctk_window = data->window;
-    CtkEvent *ctk_event = data->event;
     NvCtrlAttributeHandle *gpu_handle = data->gpu_handle;
     GtkTreeIter parent_iter = data->parent_iter;
     GtkTextTagTable *tag_table = data->tag_table;
@@ -1482,6 +1518,7 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
     gboolean parent_expanded;
     GtkTreeSelection *tree_selection =
         gtk_tree_view_get_selection(ctk_window->treeview);
+    GtkWidget *widget;
 
 
     /* Keep track if the parent row is expanded */
@@ -1496,7 +1533,6 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
     while (data->num_displays) {
 
         GtkTreeIter *iter = &(data->display_iters[data->num_displays -1]);
-        GtkWidget *widget;
 
         /* Select the parent (GPU) iter if we're removing the selected page */
         if (gtk_tree_selection_iter_is_selected(tree_selection, iter)) {
@@ -1514,10 +1550,15 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
 
         data->num_displays--;
     }
-    
+
     /* Add back all the connected display devices */
-    add_display_devices(ctk_window, &parent_iter, gpu_handle, ctk_event,
-                        tag_table, data);
+
+    gtk_tree_model_get(GTK_TREE_MODEL(ctk_window->tree_store), &parent_iter,
+                       CTK_WINDOW_WIDGET_COLUMN, &widget, -1);
+
+    add_display_devices(ctk_window, &parent_iter, gpu_handle,
+                        CTK_GPU(widget)->ctk_event,
+                        tag_table, data, ctk_window->attribute_list);
 
     /* Expand the GPU entry if it used to be */
     if (parent_expanded) {
