@@ -2,7 +2,7 @@
  * nvidia-settings: A tool for configuring the NVIDIA X driver on Unix
  * and Linux systems.
  *
- * Copyright (C) 2004 NVIDIA Corporation.
+ * Copyright (C) 2004,2012 NVIDIA Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU General Public License,
@@ -24,7 +24,13 @@
 #include "NVCtrl.h"
 #include <GL/glx.h> /* GLX #defines */
 #include <X11/extensions/Xrandr.h> /* Xrandr */
-#include <X11/extensions/Xvlib.h> /* Xv */
+
+/* Make sure we are compiling with XRandR version 1.2 or greater */
+#define MIN_RANDR_MAJOR 1
+#define MIN_RANDR_MINOR 2
+#if (RANDR_MAJOR < MIN_RANDR_MAJOR) || ((RANDR_MAJOR == MIN_RANDR_MAJOR) && (RANDR_MINOR < MIN_RANDR_MINOR))
+#error XRandR version 1.2 or greater is required.
+#endif
 
 
 #define EXTENSION_NV_CONTROL  0x1
@@ -82,6 +88,12 @@ typedef struct __NvCtrlXvBlitterAttributes NvCtrlXvBlitterAttributes;
 typedef struct __NvCtrlXvAttribute NvCtrlXvAttribute;
 typedef struct __NvCtrlXrandrAttributes NvCtrlXrandrAttributes;
 
+typedef struct {
+    float brightness[3];
+    float contrast[3];
+    float gamma[3];
+} NvCtrlGammaInput;
+
 struct __NvCtrlNvControlAttributes {
     int event_base;
     int error_base;
@@ -90,52 +102,19 @@ struct __NvCtrlNvControlAttributes {
 };
 
 struct __NvCtrlVidModeAttributes {
-    int n;
-    int sigbits;
     int major_version;
     int minor_version;
     unsigned short *lut[3];
-    float brightness[3];
-    float contrast[3];
-    float gamma[3];
-};
-
-struct __NvCtrlXvAttribute {
-    Atom atom;
-    NVCTRLAttributeValidValuesRec range;
-};
-
-struct __NvCtrlXvOverlayAttributes {
-    unsigned int port;
-    NvCtrlXvAttribute *saturation;
-    NvCtrlXvAttribute *contrast;
-    NvCtrlXvAttribute *brightness;
-    NvCtrlXvAttribute *hue;
-    NvCtrlXvAttribute *defaults;
-};
-
-struct __NvCtrlXvTextureAttributes {
-    unsigned int port;
-    NvCtrlXvAttribute *sync_to_vblank;
-    NvCtrlXvAttribute *contrast;
-    NvCtrlXvAttribute *brightness;
-    NvCtrlXvAttribute *hue;
-    NvCtrlXvAttribute *saturation;
-    NvCtrlXvAttribute *defaults;
-};
-
-struct __NvCtrlXvBlitterAttributes {
-    unsigned int port;
-    NvCtrlXvAttribute *sync_to_vblank;
-    NvCtrlXvAttribute *defaults;
+    int gammaRampSize;
+    NvCtrlGammaInput gammaInput;
 };
 
 struct __NvCtrlXvAttributes {
     unsigned int major_version;
     unsigned int minor_version;
-    NvCtrlXvOverlayAttributes *overlay; /* XVideo info (overlay) */
-    NvCtrlXvTextureAttributes *texture; /* XVideo info (texture) */
-    NvCtrlXvBlitterAttributes *blitter; /* XVideo info (blitter) */
+    Bool overlay; 
+    Bool texture;
+    Bool blitter;
 };
 
 struct __NvCtrlXrandrAttributes {
@@ -143,6 +122,10 @@ struct __NvCtrlXrandrAttributes {
     int error_base;
     int major_version;
     int minor_version;
+    Bool gammaAvailable;
+    RRCrtc gammaCrtc;
+    NvCtrlGammaInput gammaInput;
+    XRRCrtcGamma *pGammaRamp;
 };
 
 struct __NvCtrlAttributePrivateHandle {
@@ -175,21 +158,10 @@ NvCtrlInitXvAttributes (NvCtrlAttributePrivateHandle *);
 void
 NvCtrlXvAttributesClose (NvCtrlAttributePrivateHandle *);
 
-ReturnStatus
-NvCtrlXvGetAttribute (NvCtrlAttributePrivateHandle *, int, int *);
-
-ReturnStatus
-NvCtrlXvSetAttribute (NvCtrlAttributePrivateHandle *, int, int);
-
 ReturnStatus
 NvCtrlXvGetStringAttribute (NvCtrlAttributePrivateHandle *,
                            unsigned int, int, char **);
 
-ReturnStatus
-NvCtrlXvGetValidAttributeValues(NvCtrlAttributePrivateHandle *, int,
-                                NVCTRLAttributeValidValuesRec *);
-
-
 /* GLX extension attribute functions */
 
 Bool
@@ -215,30 +187,52 @@ NvCtrlInitXrandrAttributes (NvCtrlAttributePrivateHandle *);
 void
 NvCtrlXrandrAttributesClose (NvCtrlAttributePrivateHandle *);
 
-ReturnStatus
-NvCtrlXrandrGetAttribute (NvCtrlAttributePrivateHandle *, int, int *);
-
-ReturnStatus
-NvCtrlXrandrSetAttribute (NvCtrlAttributePrivateHandle *, int, int);
-
-ReturnStatus
-NvCtrlXrandrSetScreenMagicMode (NvCtrlAttributePrivateHandle *, int, int, int);
-
-ReturnStatus
-NvCtrlXrandrGetScreenMagicMode (NvCtrlAttributePrivateHandle *, int *, int *,
-                                int *);
-
 ReturnStatus
 NvCtrlXrandrGetStringAttribute (NvCtrlAttributePrivateHandle *,
                                 unsigned int, int, char **);
 
 
+ReturnStatus NvCtrlXrandrGetColorAttributes(NvCtrlAttributePrivateHandle *h,
+                                            float contrast[3],
+                                            float brightness[3],
+                                            float gamma[3]);
+
+ReturnStatus NvCtrlXrandrSetColorAttributes(NvCtrlAttributePrivateHandle *h,
+                                            float c[3],
+                                            float b[3],
+                                            float g[3],
+                                            unsigned int bitmask);
+
+ReturnStatus NvCtrlXrandrGetColorRamp(NvCtrlAttributePrivateHandle *h,
+                                      unsigned int channel,
+                                      uint16_t **lut,
+                                      int *n);
+
 /* XF86 Video Mode extension attribute functions */
 
+ReturnStatus NvCtrlVidModeGetColorAttributes(NvCtrlAttributePrivateHandle *h,
+                                             float contrast[3],
+                                             float brightness[3],
+                                             float gamma[3]);
+
+ReturnStatus NvCtrlVidModeSetColorAttributes(NvCtrlAttributePrivateHandle *h,
+                                             float c[3],
+                                             float b[3],
+                                             float g[3],
+                                             unsigned int bitmask);
+
+ReturnStatus NvCtrlVidModeGetColorRamp(NvCtrlAttributePrivateHandle *h,
+                                       unsigned int channel,
+                                       uint16_t **lut,
+                                       int *n);
+
 ReturnStatus
 NvCtrlVidModeGetStringAttribute (NvCtrlAttributePrivateHandle *,
                                    unsigned int, int, char **);
 
+ReturnStatus
+NvCtrlXrandrGetAttribute(NvCtrlAttributePrivateHandle *h,
+                         unsigned int display_mask, int attr, int64_t *val);
 
 /* Generic attribute functions */
 
@@ -286,4 +280,19 @@ NvCtrlNvControlStringOperation (NvCtrlAttributePrivateHandle *h,
                                 unsigned int display_mask, int attr,
                                 char *ptrIn, char **ptrOut);
 
+/* helper functions for XV86VidMode and RandR backends */
+
+void NvCtrlInitGammaInputStruct(NvCtrlGammaInput *pGammaInput);
+
+void NvCtrlUpdateGammaRamp(const NvCtrlGammaInput *pGammaInput,
+                           int gammaRampSize,
+                           unsigned short *gammaRamp[3],
+                           unsigned int bitmask);
+
+void NvCtrlAssignGammaInput(NvCtrlGammaInput *pGammaInput,
+                            const float inContrast[3],
+                            const float inBrightness[3],
+                            const float inGamma[3],
+                            const unsigned int bitmask);
+
 #endif /* __NVCTRL_ATTRIBUTES_PRIVATE__ */
